xquery version "3.1";

module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref";

(:~
 : Convert Module Bibref
 : *******************************************************************************************
 : This module contains functions to create bibliographical data for KMG-Texts.
 :
 : @version 2.0 (2024-10-10)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection="https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";

declare namespace functx = "http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";

declare function functx:is-node-in-sequence2
  ( $node as node()? ,
    $seq as node()* )  as xs:boolean {

   some $nodeInSeq in $seq satisfies $nodeInSeq is $node
 } ;

(:~
 : This function returns a tei:author which child tei:name which contains tei:surname and tei:forename.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $author element
 : @return tei:author
 :)
declare function bibref:make-author($author as element()) as element(tei:author)
{
    element tei:author {
        element tei:name {
            element tei:surname {
                $author/tei:name/tei:surname/text()
            },
            ', ',
            element tei:forename {
                $author/tei:name/tei:forename/text()
            }
        }
    }
};

(:~
 : This function converts the child elements of tei:title.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-nodes($node as node())
{
    typeswitch ($node)
        case text()
            return
                if (normalize-space($node) = '') then ()
                else <t>{$node}</t>
        case element(tei:hi)
            return
                if ($node[@rend="italics"]) then
                    element tei:hi {
                        for $child in $node/node()
                        return bibref:make-title-nodes($child)
                    }
                else
                    for $child in $node/node()
                    return bibref:make-title-nodes($child)
        case element(tei:q)
            return
                (if ($node/@rend="double") then <t>„</t>
                else <t>‚</t>,
                for $child in $node/node()
                    return bibref:make-title-nodes($child),
                if ($node/@rend="double") then <t>“</t>
                else <t>‘</t>)
        default
            return $node/string()
};

(:~
 : This function is the only solution to remove whitespaces while keeping hi-elements.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes node()
 : @return node()*
 :)
declare function bibref:t($nodes as node()*) as node()*
{
    for $node in $nodes
    return
      typeswitch ($node)
      case text()
      return
          ()
      case element(tei:hi)
      return
            element {QName('http://www.tei-c.org/ns/1.0', 'hi')} {
                bibref:t($node/node())
                }
      case element(i)
      return
            element i {
                bibref:t($node/node())
                }
      case element(t)
      return
          $node/text()
    case element(tei:t)
      return
          $node/text()
      default
      return ()

};

(:~
 : This function returns a tei:title.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title($title as element(tei:title)) as element(tei:title)
{
    element tei:title {
        attribute type { $title/@type },
        for $node in $title/node()
        return
            bibref:t(bibref:make-title-nodes($node))
    }
};

(:~
 : This function returns a tei:title for a Werkkommentar.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-commentary($teiHeader as element(tei:teiHeader)) as element(tei:title)
{
    let $delimiter := (
        if (ends-with($teiHeader//tei:titleStmt/tei:title[@type="main"], '?')) then ' '
        else '. '
        )
    return
        element tei:title {
            attribute type { "main" },
                let $title := (
                (<t>Werkkommentar zu KMG </t>,
                <t>{$teiHeader//tei:idno[@type='KMG']/text()}</t>,
                <t> »</t>,
                for $node in $teiHeader//tei:titleStmt/tei:title[@type="main"]/node()
                return
                    bibref:make-title-nodes($node),
                if ($teiHeader//tei:titleStmt/tei:title[@type="sub"]) then
                    <t>{$delimiter}</t>
                else (),
                if ($teiHeader//tei:titleStmt/tei:title[@type="sub"]) then
                    for $node in $teiHeader//tei:titleStmt/tei:title[@type="sub"]/node()
                    return
                        bibref:make-title-nodes($node)
                else (),
                <t>«</t>))
                return
                    bibref:t($title)
    }
};

(:~
 : This function produces the URL for one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $teiHeader tei:teiHeader
 : @param $versionToken xs:string
 : @return tei:idno
 :)
declare function bibref:make-url($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as element(tei:idno)
{
    let $baseURL := 'https://mollenhauer-edition.de/'
    return
        if ($versionToken = 'tia') then
            element tei:idno {
                attribute type { 'URL' },
                $baseURL || 'kmg.html?file=' || tokenize($teiHeader//tei:idno[@type='URI']/text(), ':')[2] || '&amp;edition=tia'
            }
        else
            element tei:idno {
                attribute type { 'URL' },
                $baseURL || 'kmg.html?file=' || tokenize($teiHeader//tei:idno[@type='URI']/text(), ':')[2] || '&amp;edition=' || $versionToken
            }
};

(:declare function bibref:make-url($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as element(tei:idno):)
(:{:)
(:    let $baseURL := 'https://mollenhauer-edition.de/works/':)
(:    return:)
(:        if ($versionToken = 'tia') then:)
(:            element tei:idno {:)
(:                attribute type { 'URL' },:)
(:                $baseURL || $teiHeader//tei:idno[@type='KMG']/text() || '-tia':)
(:            }:)
(:        else:)
(:            element tei:idno {:)
(:                attribute type { 'URL' },:)
(:                $baseURL || $teiHeader//tei:idno[@type='KMG']/text() || '-' || $versionToken:)
(:            }:)
(:};:)

(:~
 : This function produces the version description for one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $teiHeader tei:teiHeader
 : @param $versionToken xs:string
 : @return xs:string
 :)
declare function bibref:make-version-desc($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as xs:string?
{
    if ($versionToken = 'tia') then
        let $firstYearData := selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[1]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ':')[2])
        let $firstYear := (
            if ($firstYearData//fn:string[@key="date"]/text()) then
                $firstYearData//fn:string[@key="date"]/text()
            else
                $firstYearData/@zoteroID || ': ' || $firstYearData/text()
            )
        let $lastYearData := selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[last()]/tei:listWit/tei:witness[last()]/tei:bibl[last()]/@corresp, ":")[2])
        let $lastYear := (
            if ($lastYearData//fn:string[@key="date"]/text()) then
                $lastYearData//fn:string[@key="date"]/text()
            else
                $lastYearData/@zoteroID || ': ' || $lastYearData/text()
            )
        return
            'textkritisch-interaktive Ansicht ' || $firstYear || '-' || $lastYear
    else if ($versionToken = 'sta') then
            ""
(:        let $yearData := ( :)
(:            if ($teiHeader//tei:textClass/tei:keywords/tei:term[@key='documenttype:da6784cb-a922-4ddf-aca1-dc96b24f07d4']) then:)
(:                selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[last()]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2]):)
(:            else:)
(:                selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[1]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ':')[2]):)
(:            ):)
(:        let $year := ( :)
(:            if ($yearData//fn:string[@key="date"]/text()) then:)
(:                $yearData//fn:string[@key="date"]/text():)
(:            else:)
(:                $yearData/@zoteroID || ': ' || $yearData/text():)
(:            ):)
(:        return:)
(:            $year:)
    else
        $teiHeader//tei:sourceDesc/tei:listWit/tei:witness[@xml:id = $versionToken]/tei:desc/data()
};

(:~
 : This function produces a tei:biblStruct which contain bibliographical data of one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc TEI-Document
 : @param $versionToken xs:string
 : @return tei:biblStruct
 :)
declare function bibref:make-biblstruct($doc as node(), $versionToken as xs:string) as element()
{
    let $teiHeader := $doc//tei:teiHeader
    let $kmgnumber := $teiHeader//tei:idno[@type='KMG']/text()
    let $edition := element tei:idno {
                attribute type { 'editionnumber' },
                attribute kmg { $kmgnumber },
                attribute version { $versionToken },
                $kmgnumber || '-' || $versionToken
            }
    let $volumePagesURL := (
                if ($versionToken = 'sta') then
                     element tei:idno {
                        attribute type { 'pages' },
                        'Bd. X, S. Y–Z'
                    }
                else
                    bibref:make-url($teiHeader, $versionToken)
                )
    let $seriesSubTitle := (
                if ($versionToken = 'sta') then
                    element tei:title {
                            attribute type { 'sub' },
                            'Studienausgabe'
                        }
                else
                    element tei:title {
                            attribute type { 'sub' },
                            'Historisch-kritische Edition'
                        }
                )
    let $imprint := (
                if ($versionToken = 'sta') then
                    <tei:imprint>
                        <tei:pubPlace>Göttingen</tei:pubPlace>
                        <tei:publisher>Universitätsverlag</tei:publisher>
                        <tei:date>2024</tei:date>
                    </tei:imprint>
                else
                    <tei:imprint>
                        <tei:date>2024</tei:date>
                    </tei:imprint>
                )
    return
        if ($versionToken = 'tia' or $versionToken = 'sta' or $teiHeader//tei:sourceDesc/tei:listWit/tei:witness[@xml:id = $versionToken])  then
            let $authors := (
                for $author in $teiHeader//tei:titleStmt/tei:author
                return
                    bibref:make-author($author)
                )
            let $title := bibref:make-title($teiHeader//tei:titleStmt/tei:title[@type="main"])
            let $subtitle  := if ($teiHeader//tei:titleStmt/tei:title[@type="sub"]) then
                bibref:make-title($teiHeader//tei:titleStmt/tei:title[@type="sub"])
            else
                ()
            let $versionDesc := element tei:title {
                attribute type { 'versiondesc' },
                bibref:make-version-desc($teiHeader, $versionToken)
            }
            return
                element tei:biblStruct {
                    element tei:analytic {
                        $authors,
                        $edition,
                        $title,
                        $subtitle,
                        $versionDesc
                    },
                    element tei:monogr {
                        <tei:title type='main'>Klaus Mollenhauer Gesamtausgabe</tei:title>,
                        $seriesSubTitle,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Dietrich</tei:surname>, <tei:forename>Cornelie</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Horn</tei:surname>, <tei:forename>Klaus-Peter</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Müller</tei:surname>, <tei:forename>Hans-Rüdiger</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        $volumePagesURL,
                        $imprint
                    }
                }
        else if ($versionToken = 'commentary')  then
            let $authors := (
                for $editor in $teiHeader//tei:titleStmt/tei:editor
                where "Werkkommentar" = tokenize($editor/@role)
                return
                    bibref:make-author($editor)
                )
            let $title := bibref:make-title-commentary($teiHeader)
            return
                element tei:biblStruct {
                    element tei:analytic {
                        $authors,
                        $title,
                        $edition
                    },
                    element tei:monogr {
                        <tei:title type='main'>Klaus Mollenhauer Gesamtausgabe</tei:title>,
                        $seriesSubTitle,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Dietrich</tei:surname>, <tei:forename>Cornelie</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Horn</tei:surname>, <tei:forename>Klaus-Peter</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        <tei:editor>
                            <tei:name>
                                <tei:surname>Müller</tei:surname>, <tei:forename>Hans-Rüdiger</tei:forename>
                            </tei:name>
                        </tei:editor>,
                        $volumePagesURL,
                        $imprint
                    }
                }
        else
            element error {
                    attribute n {10},
                    'There is no document with KMG-number ' || $kmgnumber || ' and Fassungskürzel ' || $versionToken || ' in database'
                }
};


(:~
 : This function produces a list of tei:listBibl which contain bibliographical data of KMG-texts.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $IDs Numbers of the edition (and optionally version token)
 : @return tei:listBibl
 :)
declare function bibref:make-listbibl($IDs as xs:string*) as element(tei:listBibl)
{
    element tei:listBibl {
        for $ID in $IDs
        let $idElement := selection:TGURI-from-KMG-number(substring($ID, 1, 3))
        return
            if ($idElement/name() = 'error') then
                $idElement
            else
                let $doc := doc($config:tei || $idElement/@uri || '.xml')
                let $versionTokens := selection:dissolve-ID($ID, $doc)
                return
                    for $versionToken in $versionTokens
                    return
                        bibref:make-biblstruct($doc, $versionToken)
  }
};

(:~
 : This function produces formatted authors.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $analytic tei:analytic
 : @return xs:string
 :)
declare function bibref:format-authors($analytic as element(tei:analytic)) as xs:string?
{
    let $authors := $analytic/tei:author
    return
        if (count($authors) > 2) then
            let $number := count($authors)
            return
                let $formattedAuthors := (
                for $author at $pos in $authors
                return
                if ($pos = $number) then
                    $author/string()
                else if ($pos = ($number - 1)) then
                    $author/string() || ' &amp; '
                else
                    $author/string() || ', '
                )
                return
                    string-join($formattedAuthors)
        else if (count($authors) = 2) then
            $authors[1]/string() || ' &amp; ' || $authors[2]/string()
        else if (count($authors) = 1) then
            $authors/string()
        else
            ()
};

(:~
 : This function produces formatted authors for biblatex.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $analytic tei:analytic
 : @return xs:string
 :)
declare function bibref:format-authors-biblatex($analytic as element(tei:analytic)) as xs:string?
{
    let $authors := $analytic/tei:author
    return
        if (count($authors) > 2) then
            let $number := count($authors)
            return
                let $formattedAuthors := (
                for $author at $pos in $authors
                return
                if ($pos = $number) then
                    $author/string()
                else
                    $author/string() || ' and '
                )
                return
                    string-join($formattedAuthors)
        else if (count($authors) = 2) then
            $authors[1]/string() || ' and ' || $authors[2]/string()
        else if (count($authors) = 1) then
            $authors/string()
        else
            ()
};

(:~
 : This function produces formatted editors.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $monogr tei:monogr
 : @return xs:string
 :)
declare function bibref:format-editors($monogr) as xs:string
{
    string-join(
        (
            $monogr/tei:editor[1]//tei:forename,
            ' ',
            $monogr/tei:editor[1]//tei:surname,
            ', ',
            $monogr/tei:editor[2]//tei:forename,
            ' ',
            $monogr/tei:editor[2]//tei:surname,
            ' &amp; ',
            $monogr/tei:editor[3]//tei:forename,
            ' ',
            $monogr/tei:editor[3]//tei:surname
        )
    )
};

(:~
 : This function converts tei:title to TEI.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-nodes-html($node as node())
{
    typeswitch ($node)
        case element(tei:hi)
            return
                element i {
                    for $child in $node/node()
                    return bibref:make-title-nodes-tei($child)
(:                    <t>{$node/node()}</t>:)
                }
        default
            return
                if (normalize-space($node) = '') then ()
                else <t>{$node}</t>
};

(:~
 : This function converts tei:title to html.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-nodes-tei($node as node())
{
    typeswitch ($node)
        case element(tei:hi)
            return
                element {QName('http://www.tei-c.org/ns/1.0', 'hi')} {
                    for $child in $node/node()
                    return bibref:make-title-nodes-tei($child)
                }
        default
            return
                if (normalize-space($node) = '') then ()
                else <t>{$node}</t>
};

(:~
 : This function returns a tei:title in html.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-html($titles as element(tei:title)*)
{

    for $title in $titles
    return
        for $node in $title/node()
        return
            bibref:t(bibref:make-title-nodes-html($node))
};

(:~
 : This function returns a tei:title in TEI.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title tei:title
 : @return tei:author
 :)
declare function bibref:make-title-tei($titles as element(tei:title)*)
{
    for $title in $titles
    return
        for $node in $title/node()
        return
            bibref:t(bibref:make-title-nodes-tei($node))
};

(:~
 : This function produces formatted titles.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $analytic tei:analytic
 : @return xs:string
 :)
declare function bibref:format-title($analytic as element(tei:analytic), $format as xs:string)
{
    let $delimiter := (
        if (ends-with($analytic/tei:title[@type="main"], '?')) then '&#8239;'
        else '. '
        )
    return
        if ($analytic/tei:title[@type="sub"]) then
            (
                if ($format = "HTML") then
                    bibref:make-title-html(($analytic/tei:title[@type="main"], <tei:title>{$delimiter}</tei:title>, $analytic/tei:title[@type="sub"]))
                else
                    bibref:make-title-tei(($analytic/tei:title[@type="main"], <tei:title>{$delimiter}</tei:title>, $analytic/tei:title[@type="sub"]))
            )
        else
            if ($format = "HTML") then
                bibref:make-title-html($analytic/tei:title[@type="main"])
            else
                bibref:make-title-tei($analytic/tei:title[@type="main"])

};

declare function bibref:format-versionAndEditionNumber($biblStruct as element(tei:biblStruct)) as xs:string
{
    string-join(
            (
                '(',
                if ($biblStruct/tei:analytic/tei:title[@type="versiondesc"] = "") then ()
                else normalize-space($biblStruct/tei:analytic/tei:title[@type="versiondesc"]) || '; ',
                'KMG ',
                $biblStruct//tei:idno[@type = 'editionnumber'],
                ').'
            )
    )
};

(:~
 : This function converts a tei:listBibl to ul which contains formatted bibliographical data for KMG-texts.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $listBibl tei:listBibl
 : @return ul
 :)
declare function bibref:make-HTML($listBibl as element(tei:listBibl)) as element(ul)
{
    element ul {
        for $biblStruct in $listBibl/tei:biblStruct
        let $authors := bibref:format-authors($biblStruct/tei:analytic)
        let $title := bibref:format-title($biblStruct/tei:analytic, "HTML")
        let $versionAndEditionnumber := bibref:format-versionAndEditionNumber($biblStruct)
        let $kmg := (element i {
            string-join(
                (
                    $biblStruct/tei:monogr/tei:title[@type = 'main'],
                    '. ',
                    $biblStruct/tei:monogr/tei:title[@type = 'sub']
                )
            )
        }, '.')
        let $year := string-join(
            (
                '(',
                $biblStruct/tei:monogr//tei:date,
                ').'
            )
        )
        let $editors := bibref:format-editors($biblStruct/tei:monogr)
        let $volumePagesURL := (
            if ($biblStruct/tei:monogr/tei:idno[@type = 'pages']) then
                $biblStruct/tei:monogr/tei:idno/text()
            else
                element a {
                    attribute href { $biblStruct/tei:monogr/tei:idno/string() },
                    $biblStruct/tei:monogr/tei:idno/string()
                },
            '.'
        )
        let $publisherPlace := (
            if ($biblStruct/tei:monogr//tei:pubPlace) then
                (
                    string-join(
                        (
                            $biblStruct/tei:monogr//tei:pubPlace,
                            ': ',
                            $biblStruct/tei:monogr//tei:publisher
                        )
                    )
                )
            else
                ()
        )
        return
            if ($biblStruct//tei:idno[@type='editionnumber']/@version/data()='commentary') then
                element li {
                    if ($authors) then $authors || '. ' || $year || ' ' else(),
                    $title,
                    '. In ',
                    $kmg,
                    ' Herausgegeben von ' || $editors || ". ",
                    $volumePagesURL,
                    $publisherPlace
                }
            else
                element li {
                    if ($authors) then $authors || '. ' else  (),
                    $title,
                    ' ' || $versionAndEditionnumber || " In ",
                    $kmg,
                    $year || ' Herausgegeben von ' || $editors || ". ",
                    $volumePagesURL,
                    $publisherPlace
                }
    }
};

(:~
 : This function converts a tei:biblStruct to tei:bibl which contains formatted bibliographical data for KMG-texts.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $listBibl tei:listBibl
 : @param $type Typing of the bibls
 : @return tei:ul
 :)
declare function bibref:fullreference-TEI($biblStruct as element(tei:biblStruct), $type as xs:string?) as element(tei:bibl)
{

    let $authors := bibref:format-authors($biblStruct/tei:analytic)
    let $title := bibref:format-title($biblStruct/tei:analytic, "TEI")
    let $versionAndEditionnumber := bibref:format-versionAndEditionNumber($biblStruct)
    let $kmg := element {QName('http://www.tei-c.org/ns/1.0', 'hi')} {
        attribute rend {'italics'},
        string-join(
            (
                $biblStruct/tei:monogr/tei:title[@type = 'main'],
                '. ',
                $biblStruct/tei:monogr/tei:title[@type = 'sub']
            )
        )
    }
    let $year := string-join(
        (
            '(',
            $biblStruct/tei:monogr//tei:date,
            ').'
        )
    )
    let $editors := bibref:format-editors($biblStruct/tei:monogr)
    let $volumePagesURL := (
        if ($biblStruct/tei:monogr/tei:idno[@type = 'pages']) then
            $biblStruct/tei:monogr/tei:idno/text()
        else
            element {QName('http://www.tei-c.org/ns/1.0', 'target')} {
                attribute ref { $biblStruct/tei:monogr/tei:idno/string() },
                $biblStruct/tei:monogr/tei:idno/string()
            },
        '.'
    )
    let $publisherPlace := (
        if ($biblStruct/tei:monogr//tei:pubPlace) then
            (
                string-join(
                    (
                        $biblStruct/tei:monogr//tei:pubPlace,
                        ': ',
                        $biblStruct/tei:monogr//tei:publisher
                    )
                )
            )
        else
            ()
    )
    return

        element {QName('http://www.tei-c.org/ns/1.0', 'bibl')} {
                if ($type) then
                        attribute type {$type}
                    else
                        (),
                if ($biblStruct//tei:idno[@type='editionnumber']/@version/data()='commentary') then
                    (
                        if ($authors) then $authors || '. ' || $year || ' ' else(),
                        $title,
                        '. In ',
                        $kmg,
                        ' Herausgegeben von ' || $editors || ". ",
                        $volumePagesURL,
                        $publisherPlace
                    )
                else
                    (
                        if ($authors) then $authors || '. ' else  (),
                        $title,
                        ' ' || $versionAndEditionnumber || " In ",
                        $kmg,
                        $year || ' Herausgegeben von ' || $editors || ". ",
                        $volumePagesURL,
                        $publisherPlace
                    )
            }
};