xquery version "3.1";

module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify";

(:~
 : Convert Module Simplify TEI
 : *******************************************************************************************
 : This module contains functions to strip the TEI from its textcritical apparatus and to export a single XML-Document
 : for each witness.
 :
 : @version 1.7 (2024-10-10)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";

declare namespace tei = "http://www.tei-c.org/ns/1.0";

declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";

declare option output:method "xml";


(:~
 : Retrieves the target node of a given @copyOf-reference.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @return The node the @copyOf refers to
 :)
declare function simplify:find-corresp-node(
    $node as node(), $copyOf as xs:string)
as node()*
{
    if (fn:starts-with($copyOf, "textgrid:"))
    then
        let $uri := substring($copyOf, 10, 5)
        let $docPath := $config:sade-data || $uri || '.xml'
        let $copyOf := substring-after($copyOf, "#")
        return
            if (fn:doc-available($docPath)) then (
                let $doc := doc($docPath)
                return
                    if (fn:exists($doc//*[@xml:id = $copyOf])) then (
                        $doc//*[@xml:id = $copyOf]
                    )
                    else (
                        element span {
                            attribute class {"error-message"},
                            "Error: Can't find node with xml:id: " || $copyOf ||
                            "in text with tguri: " || $uri,
                            element br {}
                        }
                    )
            )
            else (
                element span {
                    attribute class {"error-message"},
                    "Error: Can't find corresponding text with tguri: " || $uri ||
                    " in the database. Maybe the text hasn't been been published yet?",
                    element br {}
                }
            )
    else
        let $copyOf := replace($copyOf, "#", "")
        return
            if (fn:exists($node/root()//*[@xml:id = $copyOf])) then (
                let $returningNode := $node/root()//*[@xml:id = $copyOf]
                return
                    if (count($returningNode) > 1) then (
                        let $log := util:log-system-out(util:system-time() || " " ||
                            ": Error in copyOf: There are multiple nodes with xml:id=" || $copyOf || ".")
                        return
                            $returningNode
                    )
                    else ($returningNode)
            )
            else (
                element span {
                    attribute class {"error-message"},
                    "Error: Can't find node with xml:id: " || $copyOf,
                    element br {}
                }
            )
};

(:~
 : This function resolves a tei:join according to its attributes result, scope,
 : and target.
 :
 : @param $node A tei:join
 : @return A constructed node, according to the specifications of the tei:join
 :)
declare function simplify:resolve-join($node as node(), $filename as xs:string)
as node()?
{
    let $result := $node/@result
    let $attributes := $node/@*[local-name(.) != "result" or "scope" or "target"]
    let $attributes :=
        for $attribute in $attributes return
            attribute {local-name($attribute)} {$attribute}
    let $targets := tokenize($node/@target)
    return
        element {$result} {
            $attributes,
            for $target in $targets return
                if (fn:starts-with($target, "textgrid:")) then
                    let $uri := substring($target, 10, 5)
                    let $doc := doc($config:sade-data || $uri || '.xml')
                    let $target := substring-after($target, "#")
                    return
                        if ( $node/@scope = "branches" ) then
                            ($doc//*[@xml:id = $target]/node(), " ")
                        else
                            $doc//*[@xml:id = $target]
                else
                    let $target := replace($target, "#", "")
                    let $doc := doc($config:sade-data || $filename)
                    return
                        if ( $node/@scope = "branches" ) then
                            ($doc//*[@xml:id = $target]/node(), " ")
                        else
                            $doc//*[@xml:id = $target]
        }
};

(:~
 : This function replaces all internal references of tei:name with the external links.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes: TEI-elements
 :)
declare function simplify:names($nodes as node()*)
as item ()*
{
    for $node in $nodes
    return
         typeswitch ($node)
            case text()
                return ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:name)
                return
                    if (starts-with($node/@ref, '#')) then
                        let $corresp-node := simplify:find-corresp-node($node, $node/@ref)
                        return
                            element {QName($node/namespace-uri(), $node/name())} {
                                attribute type {$node/@type},
                                if ($node/@subtype) then
                                    attribute subtype {$node/@subtype}
                                else
                                    (),
                                if ($node/@cert) then
                                    attribute cert {$node/@cert}
                                else
                                    (),
                                if ($node/@xml:id) then
                                    attribute xml:id {$node/@xml:id}
                                else
                                    (),
                                attribute ref {$corresp-node/@ref},
                                simplify:names($node/node())
                            }
                    else
                        $node
            default
                return
                    try {
                    element {QName($node/namespace-uri(), $node/name())} {
                        $node/@*,
                        simplify:names($node/node())
                    }
                } catch * {
                    simplify:names($node/node())
                }
};

(:~
 : Default return for simplify:simplify: If $node is an elment node it will be reconstructed and its child nodes will be
 : passed on to simplify:simplify. If it is not an element, its child nodes will be passed on to simplify:simplify.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:default-simplify($node as node()*)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:simplify($node/node())
        }
    } catch * {
        simplify:simplify($node/node())
    }
};

(:~
 : Return for join with @scope="branches": If $node is an elment node it will be reconstructed and its child nodes will
 : be passed on to simplify:simplify. If it is not an element, it will be passed on to simplify:simplify.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:branches-return($node as node()*)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:simplify($node/node())
        }
    } catch * {
        simplify:simplify($node)
    }
};

(:~
 : Replaces a join-element with @scope="branches" with its targets combined in one element.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:join-branches($node as node()*)
{
    let $targets := tokenize($node/@target, " ")
    let $numTargets := count($targets)
    let $conjunction := text { " " }
    return
        for $target at $count in $targets
        return
            let $corresp-node := simplify:find-corresp-node($node, $target)
            return
                if ($count = $numTargets) then
                    simplify:branches-return($corresp-node/node())
                else
                    (simplify:branches-return($corresp-node/node()), $conjunction)
};

(:~
 : This function replaces all instances of @copyOf and all join-elements with the nodes they reference to.
 : Attention: The result might be a non-valid TEI, because @xml:id might be used more than once. The result is only
 : applicable for further processing, i.e. simplify:single.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes: TEI-elements
 :)
declare function simplify:simplify($nodes as node()*)
as item ()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:join)
                return if ($node/@scope = 'root') then
                    element {QName($node/namespace-uri(), $node/@result)} {
                        for $target in tokenize($node/@target, " ")
                        return
                            let $corresp-node := simplify:find-corresp-node($node, $target)
                            return
                                simplify:default-simplify($corresp-node)
                    }
                else
                    element {QName($node/namespace-uri(), $node/@result)} {
                        simplify:join-branches($node)
                    }
            case element()
                return if (($node/@copyOf)) then
                    let $corresp-node := simplify:find-corresp-node($node, $node/@copyOf)
                    return
                        element {QName($node/namespace-uri(), $node/name())} {
                            $node/@*,
                            simplify:simplify($corresp-node/node())
                        }
                else
                    simplify:default-simplify($node)
            default
                return simplify:default-simplify($node)
};

(:~
 : Default return for simplify:single: If $node is an elment node it will be reconstructed and its child nodes will be
 : passed on to simplify:single. If it is not an element, its child nodes will be passed on to simplify:single.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:default-single($node as node()*, $fassung as xs:string)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:single($node/node(), $fassung)
        }
    } catch * {
        simplify:single($node/node(), $fassung)
    }
};

(:~
 : Default return for textcritical apparati: If $node is an elment node it will be reconstructed and its child nodes
 : will be passed on to simplify:single. If it is not an element, it will be passed on to simplify:single.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:default-app($node as node()*, $fassung as xs:string)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:single($node/node(), $fassung)
        }
    } catch * {
        simplify:single($node, $fassung)
    }
};

(:~
 : This function transform a TEI-Document without any @copyOf or join-elements into a TEI-Document without textcritical
 : apparati.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes: TEI-elements
 : @param $fassung: The selected Fassung
 :)
declare function simplify:single($nodes as node()*, $fassung as xs:string)
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:app)
                return (simplify:default-app($node/child::tei:*[contains(@wit, concat('#', $fassung))]/node(), $fassung))
            case element(tei:pb)
                return if (contains($node/@edRef, concat('#', $fassung))) then
                    ($node)
                else
                    ()
            default
                return simplify:default-single($node, $fassung)
};


(:~
 : This function calls all functions of this module in the right order.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes: TEI-elements
 : @param $fassung: The Fassung that should be converted to plain text
 :)
declare function simplify:all($nodes as node()*, $fassung as xs:string)
{
    let $names := simplify:names($nodes)
    let $simple := simplify:simplify($names)
    let $single := simplify:single($simple, $fassung)
    return
        $single
};

(:~
 : Default return for simplify:simplify-external: If $node is an elment node it will be reconstructed and its child nodes will be
 : passed on to simplify:simplify-external. If it is not an element, its child nodes will be passed on to simplify:simplify.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:default-simplify-external($node as node()*)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:simplify-external($node/node())
        }
    } catch * {
        simplify:simplify-external($node/node())
    }
};

(:~
 : Return for join with @scope="branches": If $node is an elment node it will be reconstructed and its child nodes will
 : be passed on to simplify:simplify-external. If it is not an element, it will be passed on to simplify:simplify.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:branches-return-external($node as node()*)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            simplify:simplify-external($node/node())
        }
    } catch * {
        simplify:simplify-external($node)
    }
};

(:~
 : Replaces a join-element with @scope="branches" with its targets combined in one element.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node: The current node
 :)
declare function simplify:join-branches-external($node as node()*)
{
    let $targets := tokenize($node/@target, " ")
    let $numTargets := count($targets)
    let $conjunction := text { " " }
    return
        for $target at $count in $targets
        return
            let $corresp-node := simplify:find-corresp-node($node, $target)
            return
                if ($count = $numTargets) then
                    simplify:branches-return-external($corresp-node/node())
                else
                    (simplify:branches-return-external($corresp-node/node()), $conjunction)
};

(:~
 : This function replaces all instances of @copyOf and all join-elements which reference nodes in another document with the nodes they reference to.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes: TEI-elements
 :)
declare function simplify:simplify-external($nodes as node()*)
as item ()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:join)
                return
                    let $target_sequence := tokenize($node/@target, " ")
                    let $comparison_sequence := (
                        for $target in $target_sequence
                        where starts-with($target, 'textgrid:')
                        return
                            $target
                        )
                    return
                        if (count($comparison_sequence) = count($target_sequence)) then
                            if ($node/@scope = 'root') then
                                element { QName($node/namespace-uri(), $node/@result) } {
                                    for $target in $target_sequence
                                    return
                                        let $corresp-node := simplify:find-corresp-node($node, $target)
                                        return
                                            simplify:default-simplify-external($corresp-node)
                                }
                            else
                                element { QName($node/namespace-uri(), $node/@result) } {
                                    simplify:join-branches-external($node)
                                }
                        else
                            simplify:default-simplify-external($node)
            case element()
                return if ($node/@copyOf and starts-with($node/@copyOf, 'textgrid:') and not($node/ancestor::tei:teiHeader)) then
                    let $corresp-node := simplify:find-corresp-node($node, $node/@copyOf)
                    return
                        element {QName($node/namespace-uri(), $node/name())} {
                            $node/@*[local-name() != 'copyOf'],
                            simplify:simplify-external($corresp-node/node())
                        }
                else
                    simplify:default-simplify-external($node)
            default
                return simplify:default-simplify-external($node)
};
